:-unit(t_a(A)).

% Operators for temporal reasoning

valid_args:-
	call(A).

and(X,Y):-
	call(X), call(Y).

or(X,Y):-
	call(X); call(Y).

% Time points standard operators
lesser(TP1, TP2) :-
	greater(TP2, TP1).

greater_equal(TP1, TP2) :-
	greater(TP1, TP2) ; equalTP(TP1, TP2).

lesser_equal(TP1,  TP2) :-
	greater_equal(TP2, TP1).

greater(TP1, TP2) :-
	functor(TP1, _, MAX),
	MAX1 is MAX + 1,
	greater_time_pt(TP1, TP2, 2, MAX1 ).

equalTP(TP1, TP2) :-
	functor(TP1, _, MAX),
	MAX1 is MAX+1,
	equal_time_pt(TP1, TP2, 1, MAX1).


% equal_time_pt(TP1, TP2, N):
% constraints time_pts parts of TP1 and TP1
% starting from the Nth argument to be equal
equal_time_pt(_, _, N, N):- !.

equal_time_pt(TP1, TP2, N, MAX):-
	arg(N, TP1, AN1),
	arg(N, TP2, AN2),
	AN1 #= AN2,
	N1 is N+1,
	equal_time_pt(TP1, TP2, N1, MAX).

% greater_time_pt(TP1, TP2, N):
% constraints (or checks) time_pt
% TP1 to be greater than TP2
% starting from the Nth argument
greater_time_pt(_, _, N, N):- !, fail.

greater_time_pt(TP1, TP2, P, MAX):-
	arg(P, TP1, V1),
	arg(P, TP2, V2),
	( (V1 #> V2) ;
	    ( V1 #= V2, P_N is P+1, greater_time_pt(TP1, TP2, P_N, MAX))
	).


% Allens Temporal interval predicates standard definitions
%[I1_Begin, I1_End] equal [I2_Begin, I2_End]
equalI(interval(I1_Begin, I1_End), interval(I2_Begin, I2_End)):-
	t_a( and( equalTP(I1_Begin, I2_Begin), equalTP(I1_End, I2_End))) :> valid_args.

%[I1_Begin, I1_End] overlaps [I2_Begin, I2_End]
overlaps(interval(I1_Begin, I1_End), interval(I2_Begin, I2_End)):-
	t_a( and( lesser(I1_Begin, I2_Begin), lesser(I1_End, I2_End))) :> valid_args.

overlapped_by(I1, I2):-
	overlaps(I2, I1).

%[I1_Begin, I1_End] meets [I2_Begin, I2_End]
meets(interval( _ , I1_End), interval(I2_Begin, _ )):-
	t_a( equalTP( I1_End, I2_Begin)) :> valid_args.

met_by(I1, I2):-
	meets(I2, I1).

%[I1_Begin, I1_End] before [I2_Begin, I2_End]
before(interval( _ , I1_End), interval(I2_Begin, _ )):-
	t_a( lesser(I1_End, I2_Begin)) :> valid_args.

after(I1, I2):-
	before(I2, I1).

%[I1_Begin, I1_End] during [I2_Begin, I2_End]
during(interval(I1_Begin, I1_End), interval(I2_Begin, I2_End)):-
	t_a( and( greater(I1_Begin, I2_Begin), lesser( I1_End, I2_End))) :> valid_args.

contains(I1, I2):-
	during(I2, I1).

%[I1_Begin, I1_End] starts [I2_Begin, I2_End]
starts(interval(I1_Begin, I1_End), interval(I2_Begin, I2_End)):-
	t_a( and(  equalTP(I1_Begin, I2_Begin), lesser(I1_End, I2_End))) :> valid_args.

started_by(I1, I2):-
	starts(I2, I1).

%[I1_Begin, I1_End] finishes [I2_Begin, I2_End]
finishes(interval(I1_Begin, I1_End), interval(I2_Begin, I2_End)):-
	t_a( and( greater(I1_Begin, I2_Begin), equalTP(I1_End, I2_End))) :> valid_args.

finished_by(I1, I2):-
	finishes(I2, I1).

% Time_pt element [I_Begin, I_End]
memberI(Time_pt, interval(I_Begin, I_End)):-
	t_a( and( lesser_equal(Time_pt, I_End), greater_equal(Time_pt, I_Begin))) :> valid_args.

not_memberI(Time_pt, interval(I_Begin, I_End)):-
	t_a( or( greater(Time_pt,  I_End),  lesser(Time_pt, I_Begin))) :> valid_args.
