:-unit(time_point(SELF_TP)).

% Instantiates the unit arguments
item:-
	valid_args,
	SELF_TP =.. [_|Args],
	fd_labeling(Args).

% valid unit parameters: according to gregorian calendars
% Leap year: if its multiple of 400 or if it multiple of 4 but not of 100
valid_args :-
	SELF_TP = time_point(WEEKDAY, YEAR, MONTH, DAY, HOUR, MINUTE, SEC),
	YEAR #> 0, YEAR #< 10000,
	MONTH #>= 1,  MONTH #=< 12,    % 12 months
	DAY #=< 31,
	HOUR  #>= 0,  HOUR  #=< 23,    % 24 hours
	MINUTE #>= 0, MINUTE #=< 59,   % 60 minutes
	SEC #>= 0, SEC #=< 59,   % 60 seconds
	(((( (YEAR 'rem' 400) #= 0) ##
           ( YEAR 'rem' 4 #= 0 #/\ YEAR 'rem' 100 #> 0 ))
           #/\ FEB #= 29) ##
          ( #\ (( (YEAR 'rem' 400) #= 0) ##
               ( YEAR 'rem' 4 #= 0 #/\ YEAR 'rem' 100 #> 0 ))
          #/\ FEB #=28)),
      	( ( MONTH #= 1 #/\ DAY #=< 31) ##
        ( MONTH #= 2 #/\ DAY #=< FEB) ##
        ( MONTH #= 3 #/\ DAY #=< 31) ##
        ( MONTH #= 4 #/\ DAY #=< 30) ##
        ( MONTH #= 5 #/\ DAY #=< 31) ##
        ( MONTH #= 6 #/\ DAY #=< 30) ##
        ( MONTH #= 7 #/\ DAY #=< 31) ##
        ( MONTH #= 8 #/\ DAY #=< 31) ##
        ( MONTH #= 9 #/\ DAY #=< 30) ##
        ( MONTH #= 10 #/\ DAY #=< 31) ##
        ( MONTH #= 11 #/\ DAY #=< 30) ##
        ( MONTH #= 12 #/\ DAY #=< 31)
      	),
	DAY #>= 1,
	A #= (14-MONTH)//12,
        Y #= YEAR - A,
        M #= MONTH + 12 * A - 2,
        WEEKDAY #= 1+ ((DAY + Y + Y // 4 - Y // 100 + Y //400 + 31*M//12) 'rem' 7).

% Predicates for arguments
weekday(X):-
	translate_day_number(X,N),
	arg(1, SELF_TP, N).

year(X):-
	arg(2, SELF_TP, X).

month(X):-
	arg(3, SELF_TP, X).

day(X):-
	arg(4, SELF_TP, X).

hour(X):-
	arg(5, SELF_TP, X).

minute(X):-
	arg(6, SELF_TP, X).

second(X):-
	arg(7, SELF_TP, X).

date(YEAR, MONTH, DAY):-
	year(YEAR),
	month(MONTH),
	day(DAY).

time(HOUR, MIN):-
	hour(HOUR),
	minute(MIN).

time(HOUR, MIN, SEC):-
	hour(HOUR),
	minute(MIN),
	second(SEC).


% translate_day_number: self-explanatory
translate_day_number(mon, 2).
translate_day_number(tue, 3).
translate_day_number(wed, 4).
translate_day_number(thu, 5).
translate_day_number(fri, 6).
translate_day_number(sat, 7).
translate_day_number(sun, 1).
